using System;
using System.Xml;
using System.Collections;

namespace com.ibm.gre
{
	/// <summary>
	/// Represents a single line that will be displayed in a QueryForm
	/// A Line also represents parts of the Query element
	/// in a Records Manager XML Query string.
	/// Each line is either a FieldExpression or an InterFieldOperator or
	/// one of a limited number of optional fields.
	/// </summary>
	public class Line
	{
		/// <summary>
		/// Constructor for a Line object
		/// </summary>
		/// <param name="fieldElement">The FieldValue element of a FieldExpression, or null if not used.</param>
		/// <param name="nFlPlnCmpntAttrDefDatTyp">The data type of the FieldValue, or 0 if not relevant.</param>
		/// <param name="key">The name of the XML element this line represents.</param>
		/// <param name="caption">The Caption to display.</param>
		/// <param name="pickListId">The ID of a picklist associated with the FieldValue.</param>
		/// <param name="lineType">The type of line to display. See IRMConstants.LineType.</param>
		/// <param name="controlType">The type of control that is used on this line.</param>
		/// <param name="tab">The number of tab characters appended to the line.</param>
		public Line
		(
			XmlElement fieldElement,
			int nFlPlnCmpntAttrDefDatTyp,
			string key,
			string caption,
			int pickListId,
			int lineType,
			int controlType,
			byte tab
		)
		{
			m_fieldElement = fieldElement;
			m_nFlPlnCmpntAttrDefDatTyp = nFlPlnCmpntAttrDefDatTyp;
			m_key = key;
			m_caption = caption;
			m_pickListId = pickListId;
			m_lineType = lineType;
			m_controlType = controlType;
			m_tab = tab;
		}

		/// <summary>
		/// The XML element that is associated with this line.
		/// In some cases this will be a FieldValue, Scope or optional field.
		/// </summary>
		public XmlElement FieldElement
		{
			get
			{
				return m_fieldElement;
			}
		}

		/// <summary>
		/// The data type represented on this line
		/// See IRMConstants.AttributeDataTypes
		/// </summary>
		public int FlPlnCmpntAttrDefDatTyp
		{
			get
			{
				return m_nFlPlnCmpntAttrDefDatTyp;
			}
		}

		/// <summary>
		/// The name of the XML eleement represented by this line.
		/// </summary>
		public string Key
		{
			get
			{
				return m_key;
			}
		}

		/// <summary>
		/// The caption to display for this line.
		/// </summary>
		public string Caption
		{
			get
			{
				return m_caption;
			}
		}

		/// <summary>
		/// The Picklist ID associated with the line.
		/// </summary>
		public int PickListId
		{
			get
			{
				return m_pickListId;
			}
		}

		/// <summary>
		/// The type of line that will be displayed.
		/// For example IRMConstants.LineType.SCOPE
		/// </summary>
		public int LineType
		{
			get
			{
				return m_lineType;
			}
		}

		/// <summary>
		/// The type of UI control that will be associated with the line.
		/// See IRMConstants.ProfileControlType.
		/// </summary>
		public int ControlType
		{
			get
			{
				return m_controlType;
			}
		}

		/// <summary>
		/// The number of tab characters appended to the line
		/// </summary>
		public byte Tab
		{
			get
			{
				return m_tab;
			}
		}

		private XmlElement m_fieldElement;
		private int m_nFlPlnCmpntAttrDefDatTyp;
		private string m_key;
		private string m_caption;
		private int m_pickListId;
		private int m_lineType;
		private int m_controlType;
		private byte m_tab;

	}

	/// <summary>
	/// IRMQuery class forms the basis for Easy Inteligent Query dialogs.
	/// IRMQuery objects are constructed using QueryControllerEJB saved query XML values.
	/// The class contains methods to assist in displaying Query Forms.
	/// There are methods to execute Queries with modified values.
	/// </summary>
	public class IRMQuery
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="queryXML">The XML string containing the values making up the Query.
		/// This string can be a saved query value returned by the QueryControllerEJB.getQueryById method.
		/// </param>
		/// <param name="session">A ClientSession object containing references to the Records Manager API objects.</param>
		/// <param name="strScope">The full path string indicating the scope of the query.</param>
		/// <param name="addDisplayInfo">Set to true if you wish to use this object to generate display information.</param> 
		public IRMQuery(String queryXML, ClientSession session,string strScope , bool addDisplayInfo) //, int optionalFieldsMask)
		{
			m_session = session;
			m_strScope = strScope;
			m_addDisplayInfo = addDisplayInfo;

			m_totalResults = 0;

			m_doc = new XmlDocument();
			m_doc.LoadXml(queryXML);
			m_root = m_doc.DocumentElement;
			
			m_FlPlnCmpntDefID = System.Convert.ToInt32(GetFieldValue("FlPlnCmpntDefID").InnerText);
			m_QueryNm = GetFieldValue("QueryNm").InnerText;
			//m_QueryImage = ;
			
			m_ReportTemplate = new XmlDocument();
			m_QueryImage = GetFieldValue("QueryImage");
			m_ReportTemplate.LoadXml(m_QueryImage.InnerText);
			m_rootReportTemplate = m_ReportTemplate.DocumentElement;

			m_BlockSize = (XmlElement)m_rootReportTemplate.SelectSingleNode("BlockSize");
			m_BlockStart = (XmlElement)m_rootReportTemplate.SelectSingleNode("BlockStart");
			
			m_Query = (XmlElement)m_rootReportTemplate.SelectSingleNode("Query");
			
			m_Condition = (XmlElement)m_Query.SelectSingleNode("Condition");

			m_fieldvalues = new Hashtable();

			m_DisplayItems = null;
			if(m_addDisplayInfo)
			{
				m_DisplayItems = new ArrayList();
			}

			//Get the Query Type
			m_nQueryType = System.Convert.ToInt32(GetFieldValue("QueryType").InnerText);

		}

		/// <summary>
		/// Call the Initialize method immediately after the class has been constructed
		/// to generate the internal representation of the query.
		/// </summary>
		public void Initialize()
		{
			XmlElement scope = (XmlElement)m_rootReportTemplate.SelectSingleNode("Scope");
			scope.InnerText = m_strScope;
			string caption = "Looking under the following locations"; 
			Line line = new Line(scope,(int)IRMConstants.AttributeDataTypes.STRING,"SCOPE",caption,0,(int)IRMConstants.LineType.SCOPE,(int)IRMConstants.ProfileControlType.SINGLELINE,m_tab);
			m_fieldvalues.Add("SCOPE",line);

			if(m_addDisplayInfo)
			{
				m_DisplayItems.Add(line);
			}

			//TODO optional fields used to be here

			makeProfileFields(m_Condition, m_FlPlnCmpntDefID);
			m_bIsInitialized = true;
		}

		/// <summary>
		/// Calls the QueryController runQuery using the mofified query values.
		/// Saves the result internally.
		/// Call ResultList to retreive the XML return value.
		/// </summary>
		/// <param name="nBlockStart">The index value return a block of values from. 0 indicates the first value.</param>
		/// <param name="nBlockSize">The number of values to return. 0 indicates return all values.</param> 
		/// <returns>Always returns 100 meaning 100% complete.</returns>
		public int runQuery(int nBlockStart, int nBlockSize)
		{
			BlockStart = nBlockStart;
			BlockSize = nBlockSize;
			string xml = m_session.getQueryController().runQuery(ModifiedQuery,(int)IRMConstants.QueryResultOptions.COUNT_ONLY);
			
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlElement root = doc.DocumentElement;
			m_totalResults = Convert.ToInt32(root.SelectSingleNode("Item").Attributes.GetNamedItem("RecordCount").Value);
			m_strResultList = m_session.getQueryController().runQuery(ModifiedQuery,m_nQueryResultOptions);
			return 100;
		}

		/// <summary>
		/// Get or Set the queryResultOption paramater for the QueryController runQuery method.
		/// Default value is IRMConstants.QueryResultOptions.INCLUDE_COLUMNS.
		/// </summary>
		public int QueryResultOptions
		{
			get
			{
				checkStatus();
				return m_nQueryResultOptions;
			}
			set
			{
				checkStatus();
				m_nQueryResultOptions = value;
			}
		}
		/// <summary>
		/// Return the total result count for the query
		/// </summary>
		public int TotalResults
		{
			get
			{
				checkStatus();
				return m_totalResults;
			}
		}

		/// <summary>
		/// Return the XML string result from the last call to
		/// the QueryController runQuery method.
		/// </summary>
		public string ResultList
		{
			get
			{
				checkStatus();
				return m_strResultList;
			}
		}

		/// <summary>
		/// Block Start where 0 is the first element.
		/// </summary>
		public int BlockStart
		{
			get
			{
				return System.Convert.ToInt32(m_BlockStart.InnerText);
			}
			set
			{
				m_BlockStart.InnerText = System.Convert.ToString(value);
			}
		}

		/// <summary>
		/// The number of elements to return where 0 indicates ALL.
		/// </summary>
		public int BlockSize
		{
			get
			{
				return System.Convert.ToInt32(m_BlockSize.InnerText);
			}
			set
			{
				m_BlockSize.InnerText = System.Convert.ToString(value);
			}
		}

		/// <summary>
		/// The encapsulated ClientSession
		/// </summary>
		public ClientSession ClientSession
		{
			get
			{
				return m_session;
			}
		}

		/// <summary>
		/// The Query Type.
		/// See IRMConstants.QueryTypes
		/// </summary>
		public int QueryType
		{
			get
			{
				return m_nQueryType;
			}
		}

		/// <summary>
		/// The total number of lines that will be displayed
		/// to draw a Query Form
		/// </summary>
		public int LineCount
		{
			get
			{
				checkStatus();
				checkDisplayStatus();
				if(m_DisplayItems == null)
				{
					return 0;
				}
				return m_DisplayItems.Count;
			}
		}
		
		/// <summary>
		/// Gets the Line object for the specified index.
		/// </summary>
		/// <param name="index"></param>
		/// <returns></returns>
		public Line getLine(int index)
		{
			checkStatus();
			checkDisplayStatus();
			return (Line)m_DisplayItems[index];
		}

		/// <summary>
		/// Sets the XML element value for the element named by the unique key value.
		/// </summary>
		/// <param name="key">The unique key identifying the element to set the value for.</param>
		/// <param name="val">The value to set.</param>
		/// <returns>true if the value was set.</returns>
		public bool setFieldValue(string key, string val)
		{
			checkStatus();
			if(m_fieldvalues.ContainsKey(key))
			{
				Line line = (Line)m_fieldvalues[key];
				if(validateByType(line.FlPlnCmpntAttrDefDatTyp,val))
				{
					((XmlElement)line.FieldElement).InnerText = val;
					return true;
				}
				return false;
			}
			return false;
		}

		/// <summary>
		/// Gets the string value for the XML element represented by the unique key.
		/// </summary>
		/// <param name="key">The unique value identifying the XML element to get the value for.</param>
		/// <returns>The value of the XML element or "" if not found.</returns>
		public string getFieldValue(string key)
		{
			checkStatus();
			if(m_fieldvalues.ContainsKey(key))
			{
				Line line = (Line)m_fieldvalues[key];
				return ((XmlElement)line.FieldElement).InnerText;
			}
			return "";
		}

		/// <summary>
		/// The XML string representing a new Saved Query with modified values.
		/// </summary>
		public string ModifiedQuery
		{
			get
			{
				checkStatus();
				string data = m_rootReportTemplate.OuterXml;
				m_QueryImage.ReplaceChild(m_doc.CreateCDataSection(data),m_QueryImage.FirstChild);
				return m_root.OuterXml;
			}
		}

		/// <summary>
		/// The Condition element of the saved query.
		/// </summary>
		public string Condition
		{
			get
			{
				return m_Condition.OuterXml;
			}
		}

		/// <summary>
		/// The Query element of the saved query.
		/// </summary>
		public string Query
		{
			get
			{
				return m_Query.OuterXml;
			}
		}

		/// <summary>
		/// The ReportTemplate of the saved query.
		/// </summary>
		public string ReportTemplate
		{
			get
			{	
				return m_ReportTemplate.OuterXml;
			}
		}

		/// <summary>
		/// The File Plan Component Definition Name for the saved query.
		/// </summary>
		public string FlPlnCmpntDefName
		{
			get
			{
				return getComponentName(m_FlPlnCmpntDefID);
			}
		}

		/// <summary>
		/// Validates string value according to File Plan Component Attribute
		/// Definition Type.
		/// </summary>
		/// <param name="nFlPlnCmpntAttrDefDatTyp">File Plan Component Attribute Definition Type</param>
		/// <param name="val">The value to validate</param>
		/// <returns>true if validates, false if not.</returns>
		private bool validateByType(int nFlPlnCmpntAttrDefDatTyp, string val)
		{
			try 
			{
				//TODO under construction (many more types to validate)
				switch ((IRMConstants.AttributeDataTypes)nFlPlnCmpntAttrDefDatTyp)
				{
					case IRMConstants.AttributeDataTypes.INTEGER:
						System.Convert.ToInt32(val);
						return true;
					default:
						return true;
				}
			}
			catch (System.ArgumentNullException) 
			{
				System.Console.WriteLine("String is null.");
				return false;
			}
			catch (System.FormatException) 
			{
				System.Console.WriteLine("String does not consist of an " +
					"optional sign followed by a series of digits.");
				return false;
			} 
			catch (System.OverflowException) 
			{
				System.Console.WriteLine("Overflow in string to int conversion.");
				return false;
			}
		}

		/// <summary>
		/// Recusively process all the elements contained in the Condition element
		/// and any nested Condition elements.
		/// </summary>
		/// <param name="condition">A Condition element</param>
		/// <param name="nFlPlnCmpntDefID">The Component Definition ID for the condition</param>
		private void makeProfileFields(XmlElement condition, int nFlPlnCmpntDefID)
		{
			if(m_addDisplayInfo)
			{
				string caption = "Find all the " + getComponentName(m_FlPlnCmpntDefID) + "s that meet the following criteria:"; 
				Line line = new Line(null,0,"",caption,0,(int)IRMConstants.LineType.NONE,(int)IRMConstants.ProfileControlType.GROUPHEADER,m_tab);
				m_DisplayItems.Add(line);
			}

			//Get the Columns list values so we can expand the information for Field Expressions
			string xml = m_session.getQueryController().getReportColumnsList(nFlPlnCmpntDefID);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlElement root = doc.DocumentElement;
			
			XmlNodeList list = condition.ChildNodes;
			for(int i = 0; i < list.Count; i++)
			{
				string elementName = list.Item(i).Name;
				if(elementName == "FieldExpression")
				{
					XmlElement fieldExpression = (XmlElement)list.Item(i);
					XmlElement fieldName = (XmlElement)fieldExpression.SelectSingleNode("FieldName");
					XmlElement fieldValue = (XmlElement)fieldExpression.SelectSingleNode("FieldValue");
					XmlElement fieldOperator = (XmlElement)fieldExpression.SelectSingleNode("FieldOperator");
					string strFlPlnCmpntAttrDefColNm = fieldName.InnerText;
					string key = strFlPlnCmpntAttrDefColNm + "_" + m_fieldCount++;
					
					
					string caption = getCaption(root,strFlPlnCmpntAttrDefColNm);
					
					string operation = fieldOperator.InnerText;
					int lineType = makeLineType(operation);
					operation = getOperation((IRMConstants.LineType)lineType);
					
					caption = caption + " " + operation;

					int nFlPlnCmpntAttrDefDatTyp = getDataType(root,strFlPlnCmpntAttrDefColNm);
					int nPickListId = getPickListId(root,strFlPlnCmpntAttrDefColNm);
					int nAttrCntrlMsk = getAttrCntrlMsk(root,strFlPlnCmpntAttrDefColNm);

					int controlType = (int)IRMConstants.ProfileControlType.SINGLELINE;
					if(nPickListId != 0)
					{
						if(IRMAttrCntrlMsk.isLimitToList(nAttrCntrlMsk))
						{
							controlType = (int)IRMConstants.ProfileControlType.DROPLIST;
						}
						else
						{
							controlType = (int)IRMConstants.ProfileControlType.COMBOBOX;
						}
					}
					if(lineType == (int)IRMConstants.LineType.IS_NULL)
					{
						controlType = (int)IRMConstants.ProfileControlType.BLANKLINE;
					}
					
					Line line = new Line(fieldValue,nFlPlnCmpntAttrDefDatTyp,key,caption,nPickListId,lineType,controlType,m_tab);
					
					if(lineType != (int)IRMConstants.LineType.IS_NULL)
					{
						//save the fieldValue element to receive new value
						m_fieldvalues.Add(key,line);
					}

					if(m_addDisplayInfo)
					{
						m_DisplayItems.Add(line);
					}

				}
				else if(elementName == "InterFieldOperator")
				{
					string interFieldOperator = list.Item(i).InnerText;
					int lineType = makeLineType(interFieldOperator);
					
					//Right bracket should be placed directly below preceding tab
					if(lineType == (int)IRMConstants.LineType.RB)
					{
						m_tab--;
					}
					
					Line line = new Line(null,0,"0",interFieldOperator,0,lineType,(int)IRMConstants.ProfileControlType.BLANKLINE,m_tab);
					
					//All lines after Left Bracket should be tabbed
					if(lineType == (int)IRMConstants.LineType.LB)
					{
						m_tab++;
					}

					if(m_addDisplayInfo)
					{
						m_DisplayItems.Add(line);
					}
				}
				else if(elementName == "FunctionExpression")
				{
					//FunctionExpression
					if(list.Item(i).HasChildNodes)
					{
						int localFlPlnCmpntDefID = System.Convert.ToInt32(list.Item(i).SelectSingleNode("ReportOn").InnerText);
						if(m_addDisplayInfo)
						{
							string strOperator = list.Item(i).SelectSingleNode("FunctionName").InnerText;
							int lineType = makeLineType(strOperator);
							string caption = "Filter results on " + getComponentName(localFlPlnCmpntDefID) + " " + getOperation((IRMConstants.LineType)lineType);
							Line line = new Line(null,0,"0",caption,0,lineType,(int)IRMConstants.ProfileControlType.GROUPHEADER,m_tab);
							m_DisplayItems.Add(line);
							//Add Left Bracket
							line = new Line(null,0,"0","{",0,0,(int)IRMConstants.ProfileControlType.BLANKLINE,m_tab++);
							m_DisplayItems.Add(line);
						}
						//Recusively process all the elements contained in the Condition element
						//and any nested Condition elements. 
						XmlElement localCondition = (XmlElement)list.Item(i).SelectSingleNode("Condition");
						makeProfileFields(localCondition,localFlPlnCmpntDefID);

						//Add footer item here
						if(m_addDisplayInfo)
						{
							//Add Right Bracket
							Line line = new Line(null,0,"0","}",0,0,(int)IRMConstants.ProfileControlType.BLANKLINE,--m_tab);
							m_DisplayItems.Add(line);
						}
					}
				}
			}
		}

		/// <summary>
		/// Converts a string charachter to a LineType
		/// </summary>
		/// <param name="strOperator"></param>
		/// <returns></returns>
		private int makeLineType(string strOperator)
		{
			if(strOperator == ">")
			{
				return (int)IRMConstants.LineType.GT;
			}
			else if(strOperator == "<")
			{
				return (int)IRMConstants.LineType.LT;
			}
			else if(strOperator == "=")
			{
				return (int)IRMConstants.LineType.EQ;
			}
			else if(strOperator == ">=")
			{
				return (int)IRMConstants.LineType.GTEQ;
			}
			else if(strOperator == "<=")
			{
				return (int)IRMConstants.LineType.LTEQ;
			}
			else if(strOperator == "LIKE")
			{
				return (int)IRMConstants.LineType.LIKE;
			}
			else if(strOperator == "NOT LIKE")
			{
				return (int)IRMConstants.LineType.NOT_LIKE;
			}
			else if(strOperator == "IS NULL")
			{
				return (int)IRMConstants.LineType.IS_NULL;
			}
			else if(strOperator == "(")
			{
				return (int)IRMConstants.LineType.LB;
			}
			else if(strOperator == "AND")
			{
				return (int)IRMConstants.LineType.AND;
			}
			else if(strOperator == "OR")
			{
				return (int)IRMConstants.LineType.OR;
			}
			else if(strOperator == "NOT")
			{
				return (int)IRMConstants.LineType.NOT;
			}
			else if(strOperator == ")")
			{
				return (int)IRMConstants.LineType.RB;
			}
			else if(strOperator == "hasChild")
			{
				return (int)IRMConstants.LineType.HAS_CHILD;
			}
			else if(strOperator == "isChildOf")
			{
				return (int)IRMConstants.LineType.IS_CHILD_OF;
			}
			else if(strOperator == "hasPart")
			{
				return (int)IRMConstants.LineType.HAS_PART;
			}
			else if(strOperator == "isPartOf")
			{
				return (int)IRMConstants.LineType.IS_PART_OF;
			}
			else
			{
				return (int)IRMConstants.LineType.NONE;
			}
		}

		/// <summary>
		/// Gets a character string for a line type.
		/// </summary>
		/// <param name="lt"></param>
		/// <returns></returns>
		private string getOperation(IRMConstants.LineType lt)
		{
			switch(lt)
			{
				case IRMConstants.LineType.GT:
					return "is greater than";
				case IRMConstants.LineType.LT:
					return "is less than";
				case IRMConstants.LineType.EQ:
					return "is equal to";
				case IRMConstants.LineType.GTEQ:
					return "is greater than or equal to";
				case IRMConstants.LineType.LTEQ:
					return "is less than or equal to";
				case IRMConstants.LineType.LIKE:
					return "is Like";
				case IRMConstants.LineType.NOT_LIKE:
					return "is Not Like";
				case IRMConstants.LineType.IS_NULL:
					return "has no value";
				case IRMConstants.LineType.HAS_CHILD:
					return "children matching the following conditions";
				case IRMConstants.LineType.IS_CHILD_OF:
					return "parents matching the following conditions";
				case IRMConstants.LineType.HAS_PART:
					return "that have a partition matching the following conditions";
				case IRMConstants.LineType.IS_PART_OF:
					return "that belong to a partition that matches the following conditions";
				default:
					return "";
			}
		}

		/// <summary>
		/// Helper method to extract an element value for a specified FlPlnCmpntAttrDefColNm 
		/// </summary>
		/// <param name="strFlPlnCmpntAttrDefColNm"></param>
		/// <returns></returns>
		private XmlElement GetFieldValue(string strFlPlnCmpntAttrDefColNm)
		{
			String xpath = "/Fields/Field[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
			return (XmlElement)m_root.SelectSingleNode(xpath);
		}

		/// <summary>
		/// This method gets the Caption given a column name.
		/// It is by no means optimized!
		/// Captions will be problematic in a multi language environment.
		/// There will need to be a way to map default captions to their translations 
		/// </summary>
		/// <param name="root"></param>
		/// <param name="strFlPlnCmpntAttrDefColNm"></param>
		/// <returns></returns>
		private string getCaption(XmlElement root, string strFlPlnCmpntAttrDefColNm)
		{
			String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
			XmlNode item = root.SelectSingleNode(xpath);
			string strFlPlnCmpntAttrDefCapt = item.Attributes.GetNamedItem("FlPlnCmpntAttrDefCapt").Value;
			return strFlPlnCmpntAttrDefCapt;
		}

		/// <summary>
		/// Helper method
		/// </summary>
		/// <param name="root"></param>
		/// <param name="strFlPlnCmpntAttrDefColNm"></param>
		/// <returns></returns>
		private int getDataType(XmlElement root, string strFlPlnCmpntAttrDefColNm)
		{
			String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
			XmlNode item = root.SelectSingleNode(xpath);
			string strFlPlnCmpntAttrDefDatTyp = item.Attributes.GetNamedItem("FlPlnCmpntAttrDefDatTyp").Value;
			return System.Convert.ToInt32(strFlPlnCmpntAttrDefDatTyp);
		}

		/// <summary>
		/// Helper method
		/// </summary>
		/// <param name="root"></param>
		/// <param name="strFlPlnCmpntAttrDefColNm"></param>
		/// <returns></returns>
		private int getAttrCntrlMsk(XmlElement root, string strFlPlnCmpntAttrDefColNm)
		{
			String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
			XmlNode item = root.SelectSingleNode(xpath);
			string strFlPlnCmpntAttrDefDatTyp = item.Attributes.GetNamedItem("AttrCntrlMsk").Value;
			return System.Convert.ToInt32(strFlPlnCmpntAttrDefDatTyp);
		}

		/// <summary>
		/// Helper method.
		/// </summary>
		/// <param name="root"></param>
		/// <param name="strFlPlnCmpntAttrDefColNm"></param>
		/// <returns></returns>
		private int getPickListId(XmlElement root, string strFlPlnCmpntAttrDefColNm)
		{
			String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
			XmlNode item = root.SelectSingleNode(xpath);
			string strPckLstID = item.Attributes.GetNamedItem("PckLstID").Value;
			if(strPckLstID != "")
			{
				return System.Convert.ToInt32(strPckLstID);
			}
			return 0;
		}

		/// <summary>
		/// Gets the Component Definition Name given a Component Definition ID.
		/// </summary>
		/// <param name="nFlPlnCmpntDefID"></param>
		/// <returns></returns>
		private string getComponentName(int nFlPlnCmpntDefID)
		{
			string xml = m_session.getDefinitionController().getComponentDefinitionById(nFlPlnCmpntDefID);

			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);

			XmlElement root = doc.DocumentElement;
			XmlElement item = (XmlElement)root.FirstChild;
			return item.Attributes.GetNamedItem("FlPlnCmpntDefNm").Value;
		}

		protected void checkDisplayStatus()
		{
			if(m_DisplayItems == null)
			{
				throw new Exception("Assertion: Cannot use IRM Query Display methods if the bAddDisplayInfo constructor parameter is false!");
			}
		}
	
		protected void checkStatus()
		{
			if(!m_bIsInitialized)
			{
				throw new Exception("Assertion: The Initialize method for the IRM Query object has not been called!");
			}
		}

		private int m_fieldCount = 1; //appended to field name then incremented to ensure unique field name keys		
		private int m_FlPlnCmpntDefID;
		private string m_QueryNm;
		private string m_strScope;

		private XmlDocument m_doc;
		private XmlElement m_root;
		private XmlElement m_QueryImage;
		private XmlDocument m_ReportTemplate;
		private XmlElement m_Query;
		private XmlElement m_Condition;
		private XmlElement m_BlockSize;
		private XmlElement m_BlockStart;

		protected bool m_bIsInitialized = false;

		protected XmlElement m_rootReportTemplate;

		protected byte m_tab = 0;
		protected int m_nQueryType;

		protected Hashtable m_fieldvalues;
		protected ArrayList m_DisplayItems = null;

		protected bool m_addDisplayInfo;
		protected ClientSession m_session;
		protected string m_strResultList;
		protected int m_totalResults;
		protected int m_nQueryResultOptions = (int)IRMConstants.QueryResultOptions.INCLUDE_COLUMNS;

	}
}
